# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security.settings import (
    sending_domain_restriction_edit_params,
    sending_domain_restriction_list_params,
    sending_domain_restriction_create_params,
)
from ....types.email_security.settings.sending_domain_restriction_get_response import (
    SendingDomainRestrictionGetResponse,
)
from ....types.email_security.settings.sending_domain_restriction_edit_response import (
    SendingDomainRestrictionEditResponse,
)
from ....types.email_security.settings.sending_domain_restriction_list_response import (
    SendingDomainRestrictionListResponse,
)
from ....types.email_security.settings.sending_domain_restriction_create_response import (
    SendingDomainRestrictionCreateResponse,
)
from ....types.email_security.settings.sending_domain_restriction_delete_response import (
    SendingDomainRestrictionDeleteResponse,
)

__all__ = ["SendingDomainRestrictionsResource", "AsyncSendingDomainRestrictionsResource"]


class SendingDomainRestrictionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SendingDomainRestrictionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SendingDomainRestrictionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SendingDomainRestrictionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SendingDomainRestrictionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        domain: str,
        exclude: SequenceNotStr[str],
        comments: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionCreateResponse]:
        """
        Creates a new sending domain restriction to enforce TLS requirements for a
        domain. Emails without TLS from this domain will be dropped unless the subdomain
        is in the exclude list.

        Args:
          account_id: Identifier.

          domain: Domain that requires TLS enforcement.

          exclude: Excluded subdomains that are exempt from TLS requirements.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions", account_id=account_id
            ),
            body=maybe_transform(
                {
                    "domain": domain,
                    "exclude": exclude,
                    "comments": comments,
                },
                sending_domain_restriction_create_params.SendingDomainRestrictionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionCreateResponse]],
                ResultWrapper[SendingDomainRestrictionCreateResponse],
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["domain", "created_at"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[SendingDomainRestrictionListResponse]:
        """Returns a paginated list of sending domain restrictions.

        These restrictions
        enforce TLS requirements for emails from specific domains. Mail without TLS from
        restricted domains will be dropped unless the subdomain is in the exclude list.
        Supports sorting and searching.

        Args:
          account_id: Identifier.

          direction: The sorting direction.

          order: Field to sort by.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          search: Search term for filtering records. Behavior may change.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions", account_id=account_id
            ),
            page=SyncV4PagePaginationArray[SendingDomainRestrictionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    sending_domain_restriction_list_params.SendingDomainRestrictionListParams,
                ),
            ),
            model=SendingDomainRestrictionListResponse,
        )

    def delete(
        self,
        sending_domain_restriction_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionDeleteResponse]:
        """Removes a sending domain restriction.

        After deletion, TLS will no longer be
        enforced for emails from this domain.

        Args:
          account_id: Identifier.

          sending_domain_restriction_id: Sending domain restriction identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sending_domain_restriction_id:
            raise ValueError(
                f"Expected a non-empty value for `sending_domain_restriction_id` but received {sending_domain_restriction_id!r}"
            )
        return self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions/{sending_domain_restriction_id}",
                account_id=account_id,
                sending_domain_restriction_id=sending_domain_restriction_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionDeleteResponse]],
                ResultWrapper[SendingDomainRestrictionDeleteResponse],
            ),
        )

    def edit(
        self,
        sending_domain_restriction_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        domain: str | Omit = omit,
        exclude: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionEditResponse]:
        """Updates an existing sending domain restriction.

        Only provided fields will be
        modified. Changes affect which domains require TLS and which subdomains are
        excluded.

        Args:
          account_id: Identifier.

          sending_domain_restriction_id: Sending domain restriction identifier.

          domain: Domain that requires TLS enforcement.

          exclude: Excluded subdomains that are exempt from TLS requirements.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sending_domain_restriction_id:
            raise ValueError(
                f"Expected a non-empty value for `sending_domain_restriction_id` but received {sending_domain_restriction_id!r}"
            )
        return self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions/{sending_domain_restriction_id}",
                account_id=account_id,
                sending_domain_restriction_id=sending_domain_restriction_id,
            ),
            body=maybe_transform(
                {
                    "comments": comments,
                    "domain": domain,
                    "exclude": exclude,
                },
                sending_domain_restriction_edit_params.SendingDomainRestrictionEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionEditResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionEditResponse]],
                ResultWrapper[SendingDomainRestrictionEditResponse],
            ),
        )

    def get(
        self,
        sending_domain_restriction_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionGetResponse]:
        """
        Retrieves details for a specific sending domain restriction including the domain
        requiring TLS and any excluded subdomains exempt from the TLS requirement.

        Args:
          account_id: Identifier.

          sending_domain_restriction_id: Sending domain restriction identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sending_domain_restriction_id:
            raise ValueError(
                f"Expected a non-empty value for `sending_domain_restriction_id` but received {sending_domain_restriction_id!r}"
            )
        return self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions/{sending_domain_restriction_id}",
                account_id=account_id,
                sending_domain_restriction_id=sending_domain_restriction_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionGetResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionGetResponse]], ResultWrapper[SendingDomainRestrictionGetResponse]
            ),
        )


class AsyncSendingDomainRestrictionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSendingDomainRestrictionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSendingDomainRestrictionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSendingDomainRestrictionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSendingDomainRestrictionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        domain: str,
        exclude: SequenceNotStr[str],
        comments: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionCreateResponse]:
        """
        Creates a new sending domain restriction to enforce TLS requirements for a
        domain. Emails without TLS from this domain will be dropped unless the subdomain
        is in the exclude list.

        Args:
          account_id: Identifier.

          domain: Domain that requires TLS enforcement.

          exclude: Excluded subdomains that are exempt from TLS requirements.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions", account_id=account_id
            ),
            body=await async_maybe_transform(
                {
                    "domain": domain,
                    "exclude": exclude,
                    "comments": comments,
                },
                sending_domain_restriction_create_params.SendingDomainRestrictionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionCreateResponse]],
                ResultWrapper[SendingDomainRestrictionCreateResponse],
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["domain", "created_at"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        SendingDomainRestrictionListResponse, AsyncV4PagePaginationArray[SendingDomainRestrictionListResponse]
    ]:
        """Returns a paginated list of sending domain restrictions.

        These restrictions
        enforce TLS requirements for emails from specific domains. Mail without TLS from
        restricted domains will be dropped unless the subdomain is in the exclude list.
        Supports sorting and searching.

        Args:
          account_id: Identifier.

          direction: The sorting direction.

          order: Field to sort by.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          search: Search term for filtering records. Behavior may change.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions", account_id=account_id
            ),
            page=AsyncV4PagePaginationArray[SendingDomainRestrictionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    sending_domain_restriction_list_params.SendingDomainRestrictionListParams,
                ),
            ),
            model=SendingDomainRestrictionListResponse,
        )

    async def delete(
        self,
        sending_domain_restriction_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionDeleteResponse]:
        """Removes a sending domain restriction.

        After deletion, TLS will no longer be
        enforced for emails from this domain.

        Args:
          account_id: Identifier.

          sending_domain_restriction_id: Sending domain restriction identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sending_domain_restriction_id:
            raise ValueError(
                f"Expected a non-empty value for `sending_domain_restriction_id` but received {sending_domain_restriction_id!r}"
            )
        return await self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions/{sending_domain_restriction_id}",
                account_id=account_id,
                sending_domain_restriction_id=sending_domain_restriction_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionDeleteResponse]],
                ResultWrapper[SendingDomainRestrictionDeleteResponse],
            ),
        )

    async def edit(
        self,
        sending_domain_restriction_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        domain: str | Omit = omit,
        exclude: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionEditResponse]:
        """Updates an existing sending domain restriction.

        Only provided fields will be
        modified. Changes affect which domains require TLS and which subdomains are
        excluded.

        Args:
          account_id: Identifier.

          sending_domain_restriction_id: Sending domain restriction identifier.

          domain: Domain that requires TLS enforcement.

          exclude: Excluded subdomains that are exempt from TLS requirements.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sending_domain_restriction_id:
            raise ValueError(
                f"Expected a non-empty value for `sending_domain_restriction_id` but received {sending_domain_restriction_id!r}"
            )
        return await self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions/{sending_domain_restriction_id}",
                account_id=account_id,
                sending_domain_restriction_id=sending_domain_restriction_id,
            ),
            body=await async_maybe_transform(
                {
                    "comments": comments,
                    "domain": domain,
                    "exclude": exclude,
                },
                sending_domain_restriction_edit_params.SendingDomainRestrictionEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionEditResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionEditResponse]],
                ResultWrapper[SendingDomainRestrictionEditResponse],
            ),
        )

    async def get(
        self,
        sending_domain_restriction_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SendingDomainRestrictionGetResponse]:
        """
        Retrieves details for a specific sending domain restriction including the domain
        requiring TLS and any excluded subdomains exempt from the TLS requirement.

        Args:
          account_id: Identifier.

          sending_domain_restriction_id: Sending domain restriction identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sending_domain_restriction_id:
            raise ValueError(
                f"Expected a non-empty value for `sending_domain_restriction_id` but received {sending_domain_restriction_id!r}"
            )
        return await self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/sending_domain_restrictions/{sending_domain_restriction_id}",
                account_id=account_id,
                sending_domain_restriction_id=sending_domain_restriction_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SendingDomainRestrictionGetResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[SendingDomainRestrictionGetResponse]], ResultWrapper[SendingDomainRestrictionGetResponse]
            ),
        )


class SendingDomainRestrictionsResourceWithRawResponse:
    def __init__(self, sending_domain_restrictions: SendingDomainRestrictionsResource) -> None:
        self._sending_domain_restrictions = sending_domain_restrictions

        self.create = to_raw_response_wrapper(
            sending_domain_restrictions.create,
        )
        self.list = to_raw_response_wrapper(
            sending_domain_restrictions.list,
        )
        self.delete = to_raw_response_wrapper(
            sending_domain_restrictions.delete,
        )
        self.edit = to_raw_response_wrapper(
            sending_domain_restrictions.edit,
        )
        self.get = to_raw_response_wrapper(
            sending_domain_restrictions.get,
        )


class AsyncSendingDomainRestrictionsResourceWithRawResponse:
    def __init__(self, sending_domain_restrictions: AsyncSendingDomainRestrictionsResource) -> None:
        self._sending_domain_restrictions = sending_domain_restrictions

        self.create = async_to_raw_response_wrapper(
            sending_domain_restrictions.create,
        )
        self.list = async_to_raw_response_wrapper(
            sending_domain_restrictions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sending_domain_restrictions.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            sending_domain_restrictions.edit,
        )
        self.get = async_to_raw_response_wrapper(
            sending_domain_restrictions.get,
        )


class SendingDomainRestrictionsResourceWithStreamingResponse:
    def __init__(self, sending_domain_restrictions: SendingDomainRestrictionsResource) -> None:
        self._sending_domain_restrictions = sending_domain_restrictions

        self.create = to_streamed_response_wrapper(
            sending_domain_restrictions.create,
        )
        self.list = to_streamed_response_wrapper(
            sending_domain_restrictions.list,
        )
        self.delete = to_streamed_response_wrapper(
            sending_domain_restrictions.delete,
        )
        self.edit = to_streamed_response_wrapper(
            sending_domain_restrictions.edit,
        )
        self.get = to_streamed_response_wrapper(
            sending_domain_restrictions.get,
        )


class AsyncSendingDomainRestrictionsResourceWithStreamingResponse:
    def __init__(self, sending_domain_restrictions: AsyncSendingDomainRestrictionsResource) -> None:
        self._sending_domain_restrictions = sending_domain_restrictions

        self.create = async_to_streamed_response_wrapper(
            sending_domain_restrictions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            sending_domain_restrictions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sending_domain_restrictions.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            sending_domain_restrictions.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            sending_domain_restrictions.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Workflow"]

Workflow: TypeAlias = Literal["realtime", "preview"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["TargetBulkUpdateParams", "Body", "BodyIP", "BodyIPIPV4", "BodyIPIPV6"]


class TargetBulkUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier"""

    body: Required[Iterable[Body]]


class BodyIPIPV4(TypedDict, total=False):
    """The target's IPv4 address"""

    ip_addr: str
    """IP address of the target"""

    virtual_network_id: str
    """(optional) Private virtual network identifier for the target.

    If omitted, the default virtual network ID will be used.
    """


class BodyIPIPV6(TypedDict, total=False):
    """The target's IPv6 address"""

    ip_addr: str
    """IP address of the target"""

    virtual_network_id: str
    """(optional) Private virtual network identifier for the target.

    If omitted, the default virtual network ID will be used.
    """


class BodyIP(TypedDict, total=False):
    """The IPv4/IPv6 address that identifies where to reach a target"""

    ipv4: BodyIPIPV4
    """The target's IPv4 address"""

    ipv6: BodyIPIPV6
    """The target's IPv6 address"""


class Body(TypedDict, total=False):
    hostname: Required[str]
    """A non-unique field that refers to a target.

    Case insensitive, maximum length of 255 characters, supports the use of special
    characters dash and period, does not support spaces, and must start and end with
    an alphanumeric character.
    """

    ip: Required[BodyIP]
    """The IPv4/IPv6 address that identifies where to reach a target"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from . import (
    a_record,
    ds_record,
    mx_record,
    ns_record,
    caa_record,
    loc_record,
    ptr_record,
    srv_record,
    txt_record,
    uri_record,
    aaaa_record,
    cert_record,
    svcb_record,
    tlsa_record,
    cname_record,
    https_record,
    naptr_record,
    sshfp_record,
    dnskey_record,
    smimea_record,
)
from .ttl import TTL
from ..._models import BaseModel
from .record_tags import RecordTags

__all__ = [
    "RecordResponse",
    "ARecord",
    "ARecordMeta",
    "AAAARecord",
    "AAAARecordMeta",
    "CNAMERecord",
    "CNAMERecordMeta",
    "MXRecord",
    "MXRecordMeta",
    "NSRecord",
    "NSRecordMeta",
    "OpenpgpkeyRecord",
    "OpenpgpkeyRecordMeta",
    "OpenpgpkeyRecordSettings",
    "PTRRecord",
    "PTRRecordMeta",
    "TXTRecord",
    "TXTRecordMeta",
    "CAARecord",
    "CAARecordMeta",
    "CERTRecord",
    "CERTRecordMeta",
    "DNSKEYRecord",
    "DNSKEYRecordMeta",
    "DSRecord",
    "DSRecordMeta",
    "HTTPSRecord",
    "HTTPSRecordMeta",
    "LOCRecord",
    "LOCRecordMeta",
    "NAPTRRecord",
    "NAPTRRecordMeta",
    "SMIMEARecord",
    "SMIMEARecordMeta",
    "SRVRecord",
    "SRVRecordMeta",
    "SSHFPRecord",
    "SSHFPRecordMeta",
    "SVCBRecord",
    "SVCBRecordMeta",
    "TLSARecord",
    "TLSARecordMeta",
    "URIRecord",
    "URIRecordMeta",
]


class ARecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class ARecord(a_record.ARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: ARecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class AAAARecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class AAAARecord(aaaa_record.AAAARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: AAAARecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class CNAMERecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class CNAMERecord(cname_record.CNAMERecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: CNAMERecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class MXRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class MXRecord(mx_record.MXRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: MXRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class NSRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class NSRecord(ns_record.NSRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: NSRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class OpenpgpkeyRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class OpenpgpkeyRecordSettings(BaseModel):
    """Settings for the DNS record."""

    ipv4_only: Optional[bool] = None
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: Optional[bool] = None
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class OpenpgpkeyRecord(BaseModel):
    id: str
    """Identifier."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A single Base64-encoded OpenPGP Transferable Public Key (RFC 4880 Section 11.1)"""

    created_on: datetime
    """When the record was created."""

    meta: OpenpgpkeyRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    name: str
    """Complete DNS record name, including the zone name, in Punycode."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: OpenpgpkeyRecordSettings
    """Settings for the DNS record."""

    tags: List[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""

    ttl: TTL
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Literal["OPENPGPKEY"]
    """Record type."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class PTRRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class PTRRecord(ptr_record.PTRRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: PTRRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class TXTRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class TXTRecord(txt_record.TXTRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: TXTRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class CAARecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class CAARecord(caa_record.CAARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: CAARecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class CERTRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class CERTRecord(cert_record.CERTRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: CERTRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class DNSKEYRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class DNSKEYRecord(dnskey_record.DNSKEYRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: DNSKEYRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class DSRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class DSRecord(ds_record.DSRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: DSRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class HTTPSRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class HTTPSRecord(https_record.HTTPSRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: HTTPSRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class LOCRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class LOCRecord(loc_record.LOCRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: LOCRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class NAPTRRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class NAPTRRecord(naptr_record.NAPTRRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: NAPTRRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SMIMEARecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class SMIMEARecord(smimea_record.SMIMEARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: SMIMEARecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SRVRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class SRVRecord(srv_record.SRVRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: SRVRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SSHFPRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class SSHFPRecord(sshfp_record.SSHFPRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: SSHFPRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SVCBRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class SVCBRecord(svcb_record.SVCBRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: SVCBRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class TLSARecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class TLSARecord(tlsa_record.TLSARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: TLSARecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class URIRecordMeta(BaseModel):
    """Extra Cloudflare-specific metadata about the record."""

    dead_glue: Optional[bool] = None
    """
    Whether this glue record is not served because a shallower NS delegation takes
    precedence over the deeper delegation that needs it. Present only when true;
    reachable glue carries only `is_glue`. See
    [Unreachable glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#unreachable-glue-records).
    """

    is_glue: Optional[bool] = None
    """Whether this A or AAAA record is glue for a subdomain NS delegation.

    See
    [Glue records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records#glue-records).
    """

    shadowed_by: Optional[List[str]] = None
    """IDs of the NS records that shadow this record.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """

    shadowed_records_count: Optional[int] = None
    """Number of records shadowed by this NS delegation.

    See
    [Shadowed records](https://developers.cloudflare.com/dns/manage-dns-records/reference/shadowed-records).
    """


class URIRecord(uri_record.URIRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: URIRecordMeta
    """Extra Cloudflare-specific metadata about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


RecordResponse: TypeAlias = Union[
    ARecord,
    AAAARecord,
    CNAMERecord,
    MXRecord,
    NSRecord,
    OpenpgpkeyRecord,
    PTRRecord,
    TXTRecord,
    CAARecord,
    CERTRecord,
    DNSKEYRecord,
    DSRecord,
    HTTPSRecord,
    LOCRecord,
    NAPTRRecord,
    SMIMEARecord,
    SRVRecord,
    SSHFPRecord,
    SVCBRecord,
    TLSARecord,
    URIRecord,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .relays.relays import (
    RelaysResource,
    AsyncRelaysResource,
    RelaysResourceWithRawResponse,
    AsyncRelaysResourceWithRawResponse,
    RelaysResourceWithStreamingResponse,
    AsyncRelaysResourceWithStreamingResponse,
)

__all__ = ["MoQResource", "AsyncMoQResource"]


class MoQResource(SyncAPIResource):
    @cached_property
    def relays(self) -> RelaysResource:
        return RelaysResource(self._client)

    @cached_property
    def with_raw_response(self) -> MoQResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MoQResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MoQResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MoQResourceWithStreamingResponse(self)


class AsyncMoQResource(AsyncAPIResource):
    @cached_property
    def relays(self) -> AsyncRelaysResource:
        return AsyncRelaysResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMoQResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMoQResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMoQResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMoQResourceWithStreamingResponse(self)


class MoQResourceWithRawResponse:
    def __init__(self, moq: MoQResource) -> None:
        self._moq = moq

    @cached_property
    def relays(self) -> RelaysResourceWithRawResponse:
        return RelaysResourceWithRawResponse(self._moq.relays)


class AsyncMoQResourceWithRawResponse:
    def __init__(self, moq: AsyncMoQResource) -> None:
        self._moq = moq

    @cached_property
    def relays(self) -> AsyncRelaysResourceWithRawResponse:
        return AsyncRelaysResourceWithRawResponse(self._moq.relays)


class MoQResourceWithStreamingResponse:
    def __init__(self, moq: MoQResource) -> None:
        self._moq = moq

    @cached_property
    def relays(self) -> RelaysResourceWithStreamingResponse:
        return RelaysResourceWithStreamingResponse(self._moq.relays)


class AsyncMoQResourceWithStreamingResponse:
    def __init__(self, moq: AsyncMoQResource) -> None:
        self._moq = moq

    @cached_property
    def relays(self) -> AsyncRelaysResourceWithStreamingResponse:
        return AsyncRelaysResourceWithStreamingResponse(self._moq.relays)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["CustomProviderGetResponse"]


class CustomProviderGetResponse(BaseModel):
    id: str

    base_url: str

    created_at: datetime

    modified_at: datetime

    name: str

    slug: str

    beta: Optional[bool] = None

    curl_example: Optional[str] = None

    description: Optional[str] = None

    enable: Optional[bool] = None

    headers: Optional[str] = None

    js_example: Optional[str] = None

    link: Optional[str] = None

    logo: Optional[str] = None

    position: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SetupFlowListParams"]


class SetupFlowListParams(TypedDict, total=False):
    account_id: Required[str]

    auth_method: str
    """Filter by auth method slug. Get available slugs from GET /v2/applications."""

    environment: Literal["fedramp", "standard"]
    """Filter by environment."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ZoneGetResponse"]


class ZoneGetResponse(BaseModel):
    record_quota: Optional[int] = None
    """Maximum number of DNS records allowed for the zone.

    Null if using account-level quota.
    """

    record_usage: int
    """Current number of DNS records in the zone."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SessionGenerateSummaryOfTranscriptsResponse", "Data"]


class Data(BaseModel):
    session_id: Optional[str] = None

    status: Optional[str] = None


class SessionGenerateSummaryOfTranscriptsResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ..._models import BaseModel

__all__ = ["RelayListResponse"]


class RelayListResponse(BaseModel):
    """Abbreviated relay for list responses."""

    created: datetime

    modified: datetime

    name: str

    uid: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from .evaluate import (
    EvaluateResource,
    AsyncEvaluateResource,
    EvaluateResourceWithRawResponse,
    AsyncEvaluateResourceWithRawResponse,
    EvaluateResourceWithStreamingResponse,
    AsyncEvaluateResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .flags.flags import (
    FlagsResource,
    AsyncFlagsResource,
    FlagsResourceWithRawResponse,
    AsyncFlagsResourceWithRawResponse,
    FlagsResourceWithStreamingResponse,
    AsyncFlagsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.flagship import app_create_params, app_update_params
from ....types.flagship.app_get_response import AppGetResponse
from ....types.flagship.app_list_response import AppListResponse
from ....types.flagship.app_create_response import AppCreateResponse
from ....types.flagship.app_delete_response import AppDeleteResponse
from ....types.flagship.app_update_response import AppUpdateResponse

__all__ = ["AppsResource", "AsyncAppsResource"]


class AppsResource(SyncAPIResource):
    @cached_property
    def flags(self) -> FlagsResource:
        return FlagsResource(self._client)

    @cached_property
    def evaluate(self) -> EvaluateResource:
        return EvaluateResource(self._client)

    @cached_property
    def with_raw_response(self) -> AppsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AppsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AppsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AppsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppCreateResponse:
        """Creates an app.

        The returned `id` is used in all subsequent flag, changelog, and
        evaluation requests.

        Args:
          account_id: Cloudflare account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/flagship/apps", account_id=account_id),
            body=maybe_transform({"name": name}, app_create_params.AppCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppCreateResponse], ResultWrapper[AppCreateResponse]),
        )

    def update(
        self,
        app_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppUpdateResponse:
        """Updates an app.

        Only `name` is mutable.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}", account_id=account_id, app_id=app_id),
            body=maybe_transform({"name": name}, app_update_params.AppUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppUpdateResponse], ResultWrapper[AppUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[AppListResponse]:
        """Lists all apps in the account.

        Returns identity and audit fields only — flag
        definitions are not included.

        Args:
          account_id: Cloudflare account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/flagship/apps", account_id=account_id),
            page=SyncSinglePage[AppListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=AppListResponse,
        )

    def delete(
        self,
        app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppDeleteResponse:
        """Deletes an app and all its flags and changelog history.

        Returns 409 if any
        Worker still references this app via a Flagship binding.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}", account_id=account_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppDeleteResponse], ResultWrapper[AppDeleteResponse]),
        )

    def get(
        self,
        app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppGetResponse:
        """Returns an app's name and audit fields.

        Flag definitions are not included.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}", account_id=account_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppGetResponse], ResultWrapper[AppGetResponse]),
        )


class AsyncAppsResource(AsyncAPIResource):
    @cached_property
    def flags(self) -> AsyncFlagsResource:
        return AsyncFlagsResource(self._client)

    @cached_property
    def evaluate(self) -> AsyncEvaluateResource:
        return AsyncEvaluateResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAppsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAppsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAppsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAppsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppCreateResponse:
        """Creates an app.

        The returned `id` is used in all subsequent flag, changelog, and
        evaluation requests.

        Args:
          account_id: Cloudflare account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/flagship/apps", account_id=account_id),
            body=await async_maybe_transform({"name": name}, app_create_params.AppCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppCreateResponse], ResultWrapper[AppCreateResponse]),
        )

    async def update(
        self,
        app_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppUpdateResponse:
        """Updates an app.

        Only `name` is mutable.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}", account_id=account_id, app_id=app_id),
            body=await async_maybe_transform({"name": name}, app_update_params.AppUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppUpdateResponse], ResultWrapper[AppUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AppListResponse, AsyncSinglePage[AppListResponse]]:
        """Lists all apps in the account.

        Returns identity and audit fields only — flag
        definitions are not included.

        Args:
          account_id: Cloudflare account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/flagship/apps", account_id=account_id),
            page=AsyncSinglePage[AppListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=AppListResponse,
        )

    async def delete(
        self,
        app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppDeleteResponse:
        """Deletes an app and all its flags and changelog history.

        Returns 409 if any
        Worker still references this app via a Flagship binding.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}", account_id=account_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppDeleteResponse], ResultWrapper[AppDeleteResponse]),
        )

    async def get(
        self,
        app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppGetResponse:
        """Returns an app's name and audit fields.

        Flag definitions are not included.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}", account_id=account_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AppGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AppGetResponse], ResultWrapper[AppGetResponse]),
        )


class AppsResourceWithRawResponse:
    def __init__(self, apps: AppsResource) -> None:
        self._apps = apps

        self.create = to_raw_response_wrapper(
            apps.create,
        )
        self.update = to_raw_response_wrapper(
            apps.update,
        )
        self.list = to_raw_response_wrapper(
            apps.list,
        )
        self.delete = to_raw_response_wrapper(
            apps.delete,
        )
        self.get = to_raw_response_wrapper(
            apps.get,
        )

    @cached_property
    def flags(self) -> FlagsResourceWithRawResponse:
        return FlagsResourceWithRawResponse(self._apps.flags)

    @cached_property
    def evaluate(self) -> EvaluateResourceWithRawResponse:
        return EvaluateResourceWithRawResponse(self._apps.evaluate)


class AsyncAppsResourceWithRawResponse:
    def __init__(self, apps: AsyncAppsResource) -> None:
        self._apps = apps

        self.create = async_to_raw_response_wrapper(
            apps.create,
        )
        self.update = async_to_raw_response_wrapper(
            apps.update,
        )
        self.list = async_to_raw_response_wrapper(
            apps.list,
        )
        self.delete = async_to_raw_response_wrapper(
            apps.delete,
        )
        self.get = async_to_raw_response_wrapper(
            apps.get,
        )

    @cached_property
    def flags(self) -> AsyncFlagsResourceWithRawResponse:
        return AsyncFlagsResourceWithRawResponse(self._apps.flags)

    @cached_property
    def evaluate(self) -> AsyncEvaluateResourceWithRawResponse:
        return AsyncEvaluateResourceWithRawResponse(self._apps.evaluate)


class AppsResourceWithStreamingResponse:
    def __init__(self, apps: AppsResource) -> None:
        self._apps = apps

        self.create = to_streamed_response_wrapper(
            apps.create,
        )
        self.update = to_streamed_response_wrapper(
            apps.update,
        )
        self.list = to_streamed_response_wrapper(
            apps.list,
        )
        self.delete = to_streamed_response_wrapper(
            apps.delete,
        )
        self.get = to_streamed_response_wrapper(
            apps.get,
        )

    @cached_property
    def flags(self) -> FlagsResourceWithStreamingResponse:
        return FlagsResourceWithStreamingResponse(self._apps.flags)

    @cached_property
    def evaluate(self) -> EvaluateResourceWithStreamingResponse:
        return EvaluateResourceWithStreamingResponse(self._apps.evaluate)


class AsyncAppsResourceWithStreamingResponse:
    def __init__(self, apps: AsyncAppsResource) -> None:
        self._apps = apps

        self.create = async_to_streamed_response_wrapper(
            apps.create,
        )
        self.update = async_to_streamed_response_wrapper(
            apps.update,
        )
        self.list = async_to_streamed_response_wrapper(
            apps.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            apps.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            apps.get,
        )

    @cached_property
    def flags(self) -> AsyncFlagsResourceWithStreamingResponse:
        return AsyncFlagsResourceWithStreamingResponse(self._apps.flags)

    @cached_property
    def evaluate(self) -> AsyncEvaluateResourceWithStreamingResponse:
        return AsyncEvaluateResourceWithStreamingResponse(self._apps.evaluate)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["BulkCreateParams", "SearchParams"]


class BulkCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    action: Required[Literal["MOVE", "RELEASE"]]

    search_params: Required[SearchParams]

    comment: Optional[str]

    destination: Literal["Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"]

    expected_disposition: Literal[
        "MALICIOUS", "MALICIOUS-BEC", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "ENCRYPTED", "EXTERNAL", "UNKNOWN", "NONE"
    ]


class SearchParams(TypedDict, total=False):
    action_log: bool
    """Deprecated, use `GET /investigate/{investigate_id}/action_log` instead.

    End of life: November 1, 2026.
    """

    alert_id: Optional[str]

    delivery_status: Literal["delivered", "moved", "quarantined", "rejected", "deferred", "bounced", "queued"]
    """Delivery status of the message."""

    detections_only: bool

    domain: Optional[str]

    end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """End of search date range"""

    exact_subject: Optional[str]

    final_disposition: Literal[
        "MALICIOUS", "MALICIOUS-BEC", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "ENCRYPTED", "EXTERNAL", "UNKNOWN", "NONE"
    ]

    message_action: Optional[Literal["PREVIEW", "QUARANTINE_RELEASED", "MOVED"]]

    message_id: Optional[str]

    metric: Optional[str]

    query: Optional[str]

    recipient: Optional[str]

    sender: Optional[str]

    start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Beginning of search date range"""

    subject: Optional[str]

    submissions: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OriginTLSComplianceModeUpdateResponse"]


class OriginTLSComplianceModeUpdateResponse(BaseModel):
    id: Literal["origin_tls_compliance_modes"]
    """The identifier of the caching setting."""

    editable: bool
    """Whether the setting is editable."""

    value: List[str]
    """
    List of TLS compliance modes that constrain the key-exchange algorithms
    Cloudflare may use when establishing the TLS connection to the zone's origin.
    Currently supported values are `fips` (FIPS-approved curves) and `pqh`
    (post-quantum hybrid). Future modes (e.g. `cnsa2`) may be added; clients should
    treat unknown values as opaque strings. Multiple modes are combined as the
    intersection of their permitted algorithm lists; selections whose intersection
    is empty are rejected. An empty list clears the constraint.
    """

    modified_on: Optional[datetime] = None
    """Last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SessionGetSessionParticipantDetailsResponse", "Data", "DataParticipant", "DataParticipantPeerEvent"]


class DataParticipantPeerEvent(BaseModel):
    """A connection lifecycle event recorded for a participant's peer."""

    id: Optional[str] = None
    """ID of the peer event."""

    created_at: Optional[str] = None
    """Timestamp when this peer event was created."""

    event_name: Optional[Literal["PEER_CREATED", "PEER_JOINING", "PEER_LEAVING"]] = None
    """Name of the peer event."""

    minutes_consumed: Optional[float] = None
    """Minutes consumed attributed to this event."""

    participant_id: Optional[str] = None
    """ID of the participant this event belongs to."""

    peer_id: Optional[str] = None
    """Peer ID this event belongs to."""

    preset_view_type: Optional[Literal["GROUP_CALL", "WEBINAR", "AUDIO_ROOM", "LIVESTREAM", "CHAT"]] = None
    """View type of the preset associated with the peer."""

    session_id: Optional[str] = None
    """ID of the session this event belongs to."""

    socket_session_id: Optional[str] = None
    """ID of the socket session associated with this event."""

    updated_at: Optional[str] = None
    """Timestamp when this peer event was last updated."""


class DataParticipant(BaseModel):
    id: Optional[str] = None
    """Participant ID. This maps to the corresponding peerId."""

    created_at: Optional[str] = None
    """timestamp when this participant was created."""

    custom_participant_id: Optional[str] = None
    """ID passed by client to create this participant."""

    display_name: Optional[str] = None
    """Display name of participant when joining the session."""

    duration: Optional[float] = None
    """number of minutes for which the participant was in the session."""

    joined_at: Optional[str] = None
    """timestamp at which participant joined the session."""

    left_at: Optional[str] = None
    """timestamp at which participant left the session."""

    peer_events: Optional[List[DataParticipantPeerEvent]] = None
    """Connection lifecycle events for the participant's peer.

    Only included when `include_peer_events` is true.
    """

    preset_name: Optional[str] = None
    """Name of the preset associated with the participant."""

    updated_at: Optional[str] = None
    """timestamp when this participant's data was last updated."""

    user_id: Optional[str] = None
    """User id for this participant."""


class Data(BaseModel):
    participant: Optional[DataParticipant] = None


class SessionGetSessionParticipantDetailsResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .alerting_edit_params import AlertingEditParams as AlertingEditParams
from .alerting_get_response import AlertingGetResponse as AlertingGetResponse
from .alerting_edit_response import AlertingEditResponse as AlertingEditResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["InspectGetResponse", "Error"]


class Error(BaseModel):
    """An error encountered during SPF inspection"""

    code: str
    """Error code. Known values:

    - `lookup_failed` — DNS TXT lookup failed
    - `spf_not_found` — no SPF record found
    - `invalid_spf` — record does not start with `v=spf1`
    - `invalid_domain` — PSL validation failed
    - `loop_detected` — include/redirect cycle detected
    - `invalid_mechanism` — unrecognised or malformed mechanism
    - `resource_limit_exceeded` — internal resource protection limits exceeded
      (recursion depth or query budget)
    - `max_lookups` — RFC 7208 10-lookup limit exceeded
    """

    domain: str
    """Domain where the error occurred"""

    message: str
    """Human-readable error message"""

    details: Optional[str] = None
    """Additional error-specific details (optional).

    - For `invalid_domain` errors: the invalid domain string
    - For `invalid_mechanism` errors: the invalid mechanism text (e.g.,
      "invalidmech123")
    - For `loop_detected` errors: the domain that caused the loop
    - For other error types: not present
    """


class InspectGetResponse(BaseModel):
    """Recursive SPF inspection tree"""

    components: List[object]
    """Parsed SPF components (mechanisms)"""

    domain: str
    """Domain being inspected"""

    record: str
    """Raw SPF record content"""

    total_lookups: int
    """Total number of DNS lookups performed across all includes"""

    errors: Optional[List[Error]] = None
    """
    All errors encountered during inspection, collected from the entire tree. This
    includes errors from nested includes at any depth, providing a quick overview of
    all issues without needing to traverse the nested structure. Each error includes
    a `domain` field to identify where it occurred. Empty array if no errors
    (omitted from JSON when empty).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .alerting import (
    AlertingResource,
    AsyncAlertingResource,
    AlertingResourceWithRawResponse,
    AsyncAlertingResourceWithRawResponse,
    AlertingResourceWithStreamingResponse,
    AsyncAlertingResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["CTResource", "AsyncCTResource"]


class CTResource(SyncAPIResource):
    @cached_property
    def alerting(self) -> AlertingResource:
        return AlertingResource(self._client)

    @cached_property
    def with_raw_response(self) -> CTResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CTResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CTResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CTResourceWithStreamingResponse(self)


class AsyncCTResource(AsyncAPIResource):
    @cached_property
    def alerting(self) -> AsyncAlertingResource:
        return AsyncAlertingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCTResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCTResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCTResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCTResourceWithStreamingResponse(self)


class CTResourceWithRawResponse:
    def __init__(self, ct: CTResource) -> None:
        self._ct = ct

    @cached_property
    def alerting(self) -> AlertingResourceWithRawResponse:
        return AlertingResourceWithRawResponse(self._ct.alerting)


class AsyncCTResourceWithRawResponse:
    def __init__(self, ct: AsyncCTResource) -> None:
        self._ct = ct

    @cached_property
    def alerting(self) -> AsyncAlertingResourceWithRawResponse:
        return AsyncAlertingResourceWithRawResponse(self._ct.alerting)


class CTResourceWithStreamingResponse:
    def __init__(self, ct: CTResource) -> None:
        self._ct = ct

    @cached_property
    def alerting(self) -> AlertingResourceWithStreamingResponse:
        return AlertingResourceWithStreamingResponse(self._ct.alerting)


class AsyncCTResourceWithStreamingResponse:
    def __init__(self, ct: AsyncCTResource) -> None:
        self._ct = ct

    @cached_property
    def alerting(self) -> AsyncAlertingResourceWithStreamingResponse:
        return AsyncAlertingResourceWithStreamingResponse(self._ct.alerting)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .relay_list_params import RelayListParams as RelayListParams
from .relay_get_response import RelayGetResponse as RelayGetResponse
from .relay_create_params import RelayCreateParams as RelayCreateParams
from .relay_list_response import RelayListResponse as RelayListResponse
from .relay_update_params import RelayUpdateParams as RelayUpdateParams
from .relay_create_response import RelayCreateResponse as RelayCreateResponse
from .relay_update_response import RelayUpdateResponse as RelayUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ramp import Ramp as Ramp
from .ramp_type import RampType as RampType
from .ramp_create_params import RampCreateParams as RampCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncCursorPaginationAfter, AsyncCursorPaginationAfter
from ....._base_client import AsyncPaginator, make_request_options
from .....types.flagship.apps.flags import changelog_list_params
from .....types.flagship.apps.flags.changelog_list_response import ChangelogListResponse

__all__ = ["ChangelogResource", "AsyncChangelogResource"]


class ChangelogResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChangelogResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ChangelogResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChangelogResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ChangelogResourceWithStreamingResponse(self)

    def list(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        cursor: str | Omit = omit,
        limit: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPaginationAfter[ChangelogListResponse]:
        """Returns the audit history for a flag, newest first.

        Each entry includes the
        event type and full flag state after the change; `update` entries include a
        field-level diff. Capped at 200 entries per flag.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          cursor: Pagination cursor from a previous response.

          limit: Max items to return (1–200).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}/changelog",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            page=SyncCursorPaginationAfter[ChangelogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    changelog_list_params.ChangelogListParams,
                ),
            ),
            model=cast(Any, ChangelogListResponse),  # Union types cannot be passed in as arguments in the type system
        )


class AsyncChangelogResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChangelogResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChangelogResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChangelogResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncChangelogResourceWithStreamingResponse(self)

    def list(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        cursor: str | Omit = omit,
        limit: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ChangelogListResponse, AsyncCursorPaginationAfter[ChangelogListResponse]]:
        """Returns the audit history for a flag, newest first.

        Each entry includes the
        event type and full flag state after the change; `update` entries include a
        field-level diff. Capped at 200 entries per flag.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          cursor: Pagination cursor from a previous response.

          limit: Max items to return (1–200).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}/changelog",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            page=AsyncCursorPaginationAfter[ChangelogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    changelog_list_params.ChangelogListParams,
                ),
            ),
            model=cast(Any, ChangelogListResponse),  # Union types cannot be passed in as arguments in the type system
        )


class ChangelogResourceWithRawResponse:
    def __init__(self, changelog: ChangelogResource) -> None:
        self._changelog = changelog

        self.list = to_raw_response_wrapper(
            changelog.list,
        )


class AsyncChangelogResourceWithRawResponse:
    def __init__(self, changelog: AsyncChangelogResource) -> None:
        self._changelog = changelog

        self.list = async_to_raw_response_wrapper(
            changelog.list,
        )


class ChangelogResourceWithStreamingResponse:
    def __init__(self, changelog: ChangelogResource) -> None:
        self._changelog = changelog

        self.list = to_streamed_response_wrapper(
            changelog.list,
        )


class AsyncChangelogResourceWithStreamingResponse:
    def __init__(self, changelog: AsyncChangelogResource) -> None:
        self._changelog = changelog

        self.list = async_to_streamed_response_wrapper(
            changelog.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["DatabaseGetParams"]


class DatabaseGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    fields: List[
        Literal[
            "uuid",
            "name",
            "created_at",
            "version",
            "jurisdiction",
            "num_tables",
            "file_size",
            "running_in_region",
            "read_replication",
        ]
    ]
    """Comma-separated list of fields to include in the response.

    When omitted, all fields are returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SharedQueryGetParams"]


class SharedQueryGetParams(TypedDict, total=False):
    account_id: Required[str]

    view: Literal["events", "invocations", "calculations"]
    """Select the view of the query result to return, defaults to events."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .custom_csr_list_params import CustomCsrListParams as CustomCsrListParams
from .custom_csr_get_response import CustomCsrGetResponse as CustomCsrGetResponse
from .custom_csr_create_params import CustomCsrCreateParams as CustomCsrCreateParams
from .custom_csr_list_response import CustomCsrListResponse as CustomCsrListResponse
from .custom_csr_create_response import CustomCsrCreateResponse as CustomCsrCreateResponse
from .custom_csr_delete_response import CustomCsrDeleteResponse as CustomCsrDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "TenantEntitlements",
    "AllowAddSubdomain",
    "AllowAutoAcceptInvites",
    "CNAMESetupAllowed",
    "CustomEntitlement",
    "CustomEntitlementAllocation",
    "CustomEntitlementAllocationOrganizationsAPIMaxCountAllocation",
    "CustomEntitlementAllocationOrganizationsAPIBoolAllocation",
    "CustomEntitlementAllocationOrganizationsAPINullAllocation",
    "CustomEntitlementFeature",
    "MhsCertificateCount",
    "PartialSetupAllowed",
]


class AllowAddSubdomain(BaseModel):
    type: Literal["bool"]

    value: bool


class AllowAutoAcceptInvites(BaseModel):
    type: Literal["bool"]

    value: bool


class CNAMESetupAllowed(BaseModel):
    type: Literal["bool"]

    value: bool


class CustomEntitlementAllocationOrganizationsAPIMaxCountAllocation(BaseModel):
    type: Literal["max_count"]

    value: int


class CustomEntitlementAllocationOrganizationsAPIBoolAllocation(BaseModel):
    type: Literal["bool"]

    value: bool


class CustomEntitlementAllocationOrganizationsAPINullAllocation(BaseModel):
    type: Literal[""]

    value: Optional[object] = None


CustomEntitlementAllocation: TypeAlias = Union[
    CustomEntitlementAllocationOrganizationsAPIMaxCountAllocation,
    CustomEntitlementAllocationOrganizationsAPIBoolAllocation,
    CustomEntitlementAllocationOrganizationsAPINullAllocation,
]


class CustomEntitlementFeature(BaseModel):
    key: str


class CustomEntitlement(BaseModel):
    allocation: CustomEntitlementAllocation

    feature: CustomEntitlementFeature


class MhsCertificateCount(BaseModel):
    type: Literal["max_count"]

    value: int


class PartialSetupAllowed(BaseModel):
    type: Literal["bool"]

    value: bool


class TenantEntitlements(BaseModel):
    allow_add_subdomain: AllowAddSubdomain

    allow_auto_accept_invites: AllowAutoAcceptInvites

    cname_setup_allowed: CNAMESetupAllowed

    custom_entitlements: Optional[List[CustomEntitlement]] = None

    mhs_certificate_count: MhsCertificateCount

    partial_setup_allowed: PartialSetupAllowed



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .application_list_params import ApplicationListParams as ApplicationListParams
from .integration_list_params import IntegrationListParams as IntegrationListParams
from .application_get_response import ApplicationGetResponse as ApplicationGetResponse
from .integration_get_response import IntegrationGetResponse as IntegrationGetResponse
from .application_list_response import ApplicationListResponse as ApplicationListResponse
from .integration_create_params import IntegrationCreateParams as IntegrationCreateParams
from .integration_update_params import IntegrationUpdateParams as IntegrationUpdateParams
from .integration_pause_response import IntegrationPauseResponse as IntegrationPauseResponse
from .integration_create_response import IntegrationCreateResponse as IntegrationCreateResponse
from .integration_resume_response import IntegrationResumeResponse as IntegrationResumeResponse
from .integration_update_response import IntegrationUpdateResponse as IntegrationUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = [
    "QueryCreateParams",
    "Parameters",
    "ParametersCalculation",
    "ParametersFilter",
    "ParametersFilterUnionMember0",
    "ParametersFilterWorkersObservabilityFilterLeaf",
    "ParametersGroupBy",
    "ParametersHaving",
    "ParametersNeedle",
    "ParametersOrderBy",
]


class QueryCreateParams(TypedDict, total=False):
    account_id: Required[str]

    description: Required[Optional[str]]

    name: Required[str]
    """Query name"""

    parameters: Required[Parameters]


class ParametersCalculation(TypedDict, total=False):
    operator: Required[
        Literal[
            "uniq",
            "count",
            "max",
            "min",
            "sum",
            "avg",
            "median",
            "p001",
            "p01",
            "p05",
            "p10",
            "p25",
            "p75",
            "p90",
            "p95",
            "p99",
            "p999",
            "stddev",
            "variance",
            "COUNT_DISTINCT",
            "COUNT",
            "MAX",
            "MIN",
            "SUM",
            "AVG",
            "MEDIAN",
            "P001",
            "P01",
            "P05",
            "P10",
            "P25",
            "P75",
            "P90",
            "P95",
            "P99",
            "P999",
            "STDDEV",
            "VARIANCE",
        ]
    ]

    alias: str

    key: str

    key_type: Annotated[Literal["string", "number", "boolean"], PropertyInfo(alias="keyType")]


class ParametersFilterUnionMember0(TypedDict, total=False):
    filter_combination: Required[Annotated[Literal["and", "or", "AND", "OR"], PropertyInfo(alias="filterCombination")]]

    filters: Required[Iterable[object]]

    kind: Required[Literal["group"]]


class ParametersFilterWorkersObservabilityFilterLeaf(TypedDict, total=False):
    """A filter condition applied to query results.

    Use the keys and values endpoints to discover available fields and their values before constructing filters.
    """

    key: Required[str]
    """Filter field name.

    Use verified keys from previous query results or the keys endpoint. Common keys
    include $metadata.service, $metadata.origin, $metadata.trigger,
    $metadata.message, and $metadata.error.
    """

    operation: Required[
        Literal[
            "includes",
            "not_includes",
            "starts_with",
            "ends_with",
            "regex",
            "exists",
            "is_null",
            "in",
            "not_in",
            "eq",
            "neq",
            "gt",
            "gte",
            "lt",
            "lte",
            "=",
            "!=",
            ">",
            ">=",
            "<",
            "<=",
            "INCLUDES",
            "DOES_NOT_INCLUDE",
            "MATCH_REGEX",
            "EXISTS",
            "DOES_NOT_EXIST",
            "IN",
            "NOT_IN",
            "STARTS_WITH",
            "ENDS_WITH",
        ]
    ]
    """Comparison operator.

    String operators: includes, not_includes, starts_with, ends_with, regex.
    Existence: exists, is_null. Set membership: in, not_in (comma-separated values).
    Numeric: eq, neq, gt, gte, lt, lte.
    """

    type: Required[Literal["string", "number", "boolean"]]
    """Data type of the filter field.

    Must match the actual type of the key being filtered.
    """

    kind: Literal["filter"]
    """Discriminator for leaf filter nodes.

    Always 'filter' when present; may be omitted.
    """

    value: Union[str, float, bool]
    """Comparison value.

    Must match actual values in your data — verify with the values endpoint. Ensure
    the value type (string/number/boolean) matches the field type. String
    comparisons are case-sensitive. Regex uses RE2 syntax (no
    lookaheads/lookbehinds).
    """


ParametersFilter: TypeAlias = Union[ParametersFilterUnionMember0, ParametersFilterWorkersObservabilityFilterLeaf]


class ParametersGroupBy(TypedDict, total=False):
    type: Required[Literal["string", "number", "boolean"]]

    value: Required[str]


class ParametersHaving(TypedDict, total=False):
    key: Required[str]

    operation: Required[Literal["eq", "neq", "gt", "gte", "lt", "lte"]]

    value: Required[float]


class ParametersNeedle(TypedDict, total=False):
    """Define an expression to search using full-text search."""

    value: Required[Union[str, float, bool]]

    is_regex: Annotated[bool, PropertyInfo(alias="isRegex")]

    match_case: Annotated[bool, PropertyInfo(alias="matchCase")]


class ParametersOrderBy(TypedDict, total=False):
    """Configure the order of the results returned by the query."""

    value: Required[str]
    """Configure which Calculation to order the results by."""

    order: Literal["asc", "desc"]
    """Set the order of the results"""


class Parameters(TypedDict, total=False):
    calculations: Iterable[ParametersCalculation]
    """Create Calculations to compute as part of the query."""

    datasets: SequenceNotStr[str]
    """Set the Datasets to query. Leave it empty to query all the datasets."""

    filter_combination: Annotated[Literal["and", "or", "AND", "OR"], PropertyInfo(alias="filterCombination")]
    """Set a Flag to describe how to combine the filters on the query."""

    filters: Iterable[ParametersFilter]
    """Configure the Filters to apply to the query.

    Supports nested groups via kind: 'group'.
    """

    group_bys: Annotated[Iterable[ParametersGroupBy], PropertyInfo(alias="groupBys")]
    """Define how to group the results of the query."""

    havings: Iterable[ParametersHaving]
    """Configure the Having clauses that filter on calculations in the query result."""

    limit: int
    """Set a limit on the number of results / records returned by the query"""

    needle: ParametersNeedle
    """Define an expression to search using full-text search."""

    order_by: Annotated[ParametersOrderBy, PropertyInfo(alias="orderBy")]
    """Configure the order of the results returned by the query."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from ..fallback_domain import FallbackDomain
from ..split_tunnel_exclude import SplitTunnelExclude
from ..split_tunnel_include import SplitTunnelInclude

__all__ = ["DefaultGetResponse", "DNSSearchSuffix", "GlobalAcceleration", "ServiceModeV2", "VirtualNetworks"]


class DNSSearchSuffix(BaseModel):
    suffix: str
    """The DNS search suffix to append when resolving short hostnames."""

    description: Optional[str] = None
    """A description of the DNS search suffix."""


class GlobalAcceleration(BaseModel):
    """Global Acceleration settings for China.

    When configured, WARP clients connect to the Global Accelerator addresses instead of the default ones. Please contact your account representative to enable this feature on your account. See https://developers.cloudflare.com/china-network/concepts/global-acceleration/.
    """

    api_endpoints: List[str]
    """IP:port entries for the API endpoints."""

    enabled: bool
    """Global acceleration settings are used only when "enabled"."""

    masque_endpoints: List[str]
    """IP:port entries for the MASQUE tunnel endpoints.

    Either wireguard_endpoints or masque_endpoints must be provided.
    """

    wireguard_endpoints: List[str]
    """IP:port entries for the WireGuard tunnel endpoints.

    Either wireguard_endpoints or masque_endpoints must be provided.
    """


class ServiceModeV2(BaseModel):
    mode: Optional[str] = None
    """The mode to run the WARP client under."""

    port: Optional[float] = None
    """The port number when used with proxy mode."""


class VirtualNetworks(BaseModel):
    """Virtual network access settings for the device."""

    allowed: List[str]
    """List of virtual network IDs the device is allowed to access.

    When virtual_networks is set, at least one entry is required.
    """

    default: str
    """The default virtual network ID. Must be included in the `allowed` list."""


class DefaultGetResponse(BaseModel):
    allow_mode_switch: Optional[bool] = None
    """Whether to allow the user to switch WARP between modes."""

    allow_updates: Optional[bool] = None
    """
    Whether to receive update notifications when a new version of the client is
    available.
    """

    allowed_to_leave: Optional[bool] = None
    """Whether to allow devices to leave the organization."""

    auto_connect: Optional[float] = None
    """The amount of time in seconds to reconnect after having been disabled."""

    captive_portal: Optional[float] = None
    """Turn on the captive portal after the specified amount of time."""

    default: Optional[bool] = None
    """Whether the policy will be applied to matching devices."""

    disable_auto_fallback: Optional[bool] = None
    """
    If the `dns_server` field of a fallback domain is not present, the client will
    fall back to a best guess of the default/system DNS resolvers unless this policy
    option is set to `true`.
    """

    dns_search_suffixes: Optional[List[DNSSearchSuffix]] = None
    """List of DNS search suffixes to apply to clients.

    Suffixes are evaluated in order. Use an empty array to clear.
    """

    enabled: Optional[bool] = None
    """Whether the policy will be applied to matching devices."""

    exclude: Optional[List[SplitTunnelExclude]] = None
    """List of routes excluded in the WARP client's tunnel."""

    exclude_office_ips: Optional[bool] = None
    """Whether to add Microsoft IPs to Split Tunnel exclusions."""

    fallback_domains: Optional[List[FallbackDomain]] = None

    gateway_unique_id: Optional[str] = None

    global_acceleration: Optional[GlobalAcceleration] = None
    """Global Acceleration settings for China.

    When configured, WARP clients connect to the Global Accelerator addresses
    instead of the default ones. Please contact your account representative to
    enable this feature on your account. See
    https://developers.cloudflare.com/china-network/concepts/global-acceleration/.
    """

    include: Optional[List[SplitTunnelInclude]] = None
    """List of routes included in the WARP client's tunnel."""

    policy_id: Optional[str] = None

    register_interface_ip_with_dns: Optional[bool] = None
    """
    Determines if the operating system will register WARP's local interface IP with
    your on-premises DNS server.
    """

    sccm_vpn_boundary_support: Optional[bool] = None
    """
    Determines whether the WARP client indicates to SCCM that it is inside a VPN
    boundary. (Windows only).
    """

    service_mode_v2: Optional[ServiceModeV2] = None

    support_url: Optional[str] = None
    """The URL to launch when the Send Feedback button is clicked."""

    switch_locked: Optional[bool] = None
    """
    Whether to allow the user to turn off the WARP switch and disconnect the client.
    """

    tunnel_protocol: Optional[str] = None
    """Determines which tunnel protocol to use."""

    virtual_networks: Optional[VirtualNetworks] = None
    """Virtual network access settings for the device."""
